# © 2021 Le Filament (<http://www.le-filament.com>)
# © 2021 Confédération Générale des Scop (<https://www.les-scop.coop>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from psycopg2.extensions import AsIs

from odoo import api, fields, models, tools


class ScopPartnerActivity(models.Model):
    _name = "scop.partner.activity"
    _description = "Activité par partner"
    _order = "year desc"
    _auto = False

    # Vue
    partner_id = fields.Many2one("res.partner", string="Coopérative", readonly=True)
    year = fields.Char("Année", readonly=True)
    amount_dev = fields.Integer("Développement (hrs)", readonly=True)
    amount_acc = fields.Integer("Suivi (hrs)", readonly=True)
    amount_rev = fields.Integer("Révision (hrs)", readonly=True)
    amount_for = fields.Integer("Formation (hrs)", readonly=True)

    # ------------------------------------------------------
    # Construction de la requete
    # ------------------------------------------------------
    @api.model
    def _select(self):
        query = """
                SELECT
                row_number() OVER () as id,
                wrk.partner_id as partner_id,
                wrk.year::varchar(10) as year,
                sum(wrk.amount_rev) as amount_rev,
                sum(wrk.amount_dev) as amount_dev,
                sum(wrk.amount_acc) as amount_acc,
                sum(wrk.amount_for) as amount_for
                from (
                select
                ac.partner_id as partner_id,
                EXTRACT(YEAR FROM ac.date) as "year",
                ac.unit_amount as amount_rev,
                0 as amount_dev,
                0 as amount_acc,
                0 as amount_for
                from
                account_analytic_line as ac
                left join project_project as pu on pu.id = ac.project_id
                left join cgscop_timesheet_code as pn on pn.id = pu.cgscop_timesheet_code_id
                where
                pn.domain = 'R'
                union all
                select
                ac.partner_id as partner_id,
                EXTRACT(YEAR FROM ac.date) as "year",
                0 as amount_rev,
                ac.unit_amount as amount_dev,
                0 as amount_acc,
                0 as amount_for
                from
                account_analytic_line as ac
                left join project_project as pu on pu.id = ac.project_id
                left join cgscop_timesheet_code as pn on pn.id = pu.cgscop_timesheet_code_id
                where
                pn.domain = 'D'
                union all
                select
                ac.partner_id as partner_id,
                EXTRACT(YEAR FROM ac.date) as "year",
                0 as amount_rev,
                0 as amount_dev,
                ac.unit_amount as amount_acc,
                0 as amount_for
                from
                account_analytic_line as ac
                left join project_project as pu on pu.id = ac.project_id
                left join cgscop_timesheet_code as pn on pn.id = pu.cgscop_timesheet_code_id
                where
                pn.domain = 'A'
                union all
                select
                ac.partner_id as partner_id,
                EXTRACT(YEAR FROM ac.date) as "year",
                0 as amount_rev,
                0 as amount_dev,
                0 as amount_acc,
                ac.unit_amount as amount_for
                from
                account_analytic_line as ac
                left join project_project as pu on pu.id = ac.project_id
                left join cgscop_timesheet_code as pn on pn.id = pu.cgscop_timesheet_code_id
                where
                pn.domain = 'F'
                ) as wrk
                group BY
                wrk.partner_id,
                wrk.year
                """
        return query

    def init(self):
        tools.drop_view_if_exists(self.env.cr, self._table)
        self.env.cr.execute(
            "CREATE or REPLACE VIEW %s as (%s)",
            (AsIs(self._table), AsIs(self._select())),
        )
