# © 2021 Le Filament (<http://www.le-filament.com>)
# © 2021 Confédération Générale des Scop (<https://www.les-scop.coop>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import datetime

from odoo import fields, models


class CgScopPartnerCrm(models.Model):
    _inherit = "res.partner"

    partner_activity_ids = fields.One2many(
        comodel_name="scop.partner.activity",
        inverse_name="partner_id",
        string="Activité",
        readonly=True,
    )

    followup_format_id = fields.Many2one(
        "scop.followup.format", string="Format de suivi", ondelete="restrict"
    )

    followup_duree = fields.Integer(
        "Durée thérorique du suivi annuel",
        related="followup_format_id.duree",
        store=False,
    )

    followup_current_year = fields.Integer(
        "Durée du suivi année en cours",
        compute="_compute_followup_current_year",
        store=False,
    )

    # ------------------------------------------------------
    # Champs calculés durée du suivi de l'année en cours
    # ------------------------------------------------------
    def _compute_followup_current_year(self):
        for rec in self:

            # bilan de l'activité de l'année en cours
            wyear = str(datetime.datetime.today().year)
            acts = rec.partner_activity_ids.filtered(lambda i: i.year == wyear)

            if len(acts) <= 0:
                rec.followup_current_year = 0
            else:
                rec.followup_current_year = acts[0].amount_acc
